//@author: motzi
//@help: creates a tube-like strip of triangles thickness

#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "Rope", Category = "2D", Help = "Creates triangle strip coordinates from vertices", Tags = "", Author = "motzi")]
	#endregion PluginInfo
	public class ValueTriangleRopeNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Points", DefaultValue = 1.0)]
		public ISpread<Vector2D> FInput;
		
		[Input("Thickness", DefaultValue = 0.2)]
		public ISpread<double> FWidth;

		[Output("Vertices")]
		public ISpread<Vector2D> FOutput;
		
		[Output("Texture Coordinates")]
		public ISpread<Vector2D> FTexCoords;
		
		[Output("Factor", Visibility = PinVisibility.OnlyInspector)]
		public ISpread<double> IFactor;

		//[Import()]
		//public ILogger FLogger;

		#endregion fields & pins
		
		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			// no vertices
			if(FInput.SliceCount < 1)
			{
				return;
			}
			// Only one vertex
			else if(FInput.SliceCount == 1){
				FOutput.SliceCount = FInput.SliceCount;
				FOutput[0] = FInput[0];
				//FLogger.Log(LogType.Debug, "Single");
				
				IFactor[0] = 1;
				return;
			}

			IFactor[0] = 1.0 /(FInput.SliceCount-1);
			
			FOutput.SliceCount = FInput.SliceCount * 2;
			FTexCoords.SliceCount = FInput.SliceCount * 2;
			
			Vector2D hn;
			int i;
			
			
			for (i = 1; i < FInput.SliceCount; i++)
			{
				hn = GetHalfNormal(FInput[i], FInput[i-1], FWidth[i-1]);
				FOutput[(i-1)*2] = FInput[i-1] + hn;
				FTexCoords[(i-1)*2] = new Vector2D(IFactor[0] * (i-1),0);
				
				FOutput[(i-1)*2 + 1] = FInput[i-1] - hn;
				FTexCoords[(i-1)*2 + 1] = new Vector2D(IFactor[0] * (i-1),1);
			}
			
			--i;
			hn = GetHalfNormal(FInput[i], FInput[i-1], FWidth[i]);
			FOutput[i*2] = FInput[i] + hn;
			FTexCoords[i*2] = new Vector2D(1,0);
			
			FOutput[i*2+1] = FInput[i] - hn;
			FTexCoords[i*2+1] = new Vector2D(1,1);
			
		
		}
		
		private Vector2D GetHalfNormal(Vector2D vertexOne, Vector2D vertexTwo, double width)
		{
			Vector2D halfnormal = vertexTwo - vertexOne;
			halfnormal = ~halfnormal;
			halfnormal = new Vector2D(-halfnormal.y  * width * 0.5, halfnormal.x * width * 0.5);
			
			return halfnormal;
		}
	}
}
